@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008, Nokia
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -25,7 +25,6 @@
 #include <gio/gio.h>
 
 #include <libtracker-extract/tracker-extract.h>
-#include <libtracker-extract/tracker-utils.h>
 
 #undef  TRY_LOCALE_TO_UTF8_CONVERSION
 
@@ -37,13 +36,7 @@ static void extract_text (const gchar          *uri,
                           TrackerSparqlBuilder *metadata);
 
 static TrackerExtractData data[] = {
-	{ "text/plain",       extract_text },
-	{ "text/x-authors",   extract_text },
-	{ "text/x-changelog", extract_text },
-	{ "text/x-copying",   extract_text },
-	{ "text/x-credits",   extract_text },
-	{ "text/x-install",   extract_text },
-	{ "text/x-readme",    extract_text },
+	{ "text/*", extract_text },
 	{ NULL, NULL }
 };
 
